package net.libertacasa.pubsh.web;

import java.security.Principal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;

import org.keycloak.KeycloakPrincipal;
import org.keycloak.KeycloakSecurityContext;
import org.keycloak.adapters.KeycloakDeployment;
import org.keycloak.adapters.RefreshableKeycloakSecurityContext;
import org.keycloak.adapters.springsecurity.token.KeycloakAuthenticationToken;
import org.keycloak.representations.IDToken;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.http.MediaType;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.DeleteMapping;
//import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
//import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.github.dockerjava.api.model.Container;
import com.github.dockerjava.api.model.Image;
import java.util.Random;

//@SessionAttributes("osChoice")
@SpringBootApplication
@Controller
public class WebApplication {

	public static void main(String[] args) {
		SpringApplication.run(WebApplication.class, args);
	}

	@GetMapping("/")
	public String index() {
		return "external";
		
	}
	
	@GetMapping("/portal")
	public String portal(Model model) {
		KeycloakAuthenticationToken authentication = (KeycloakAuthenticationToken)
        SecurityContextHolder.getContext().getAuthentication();
        
		Principal principal = (Principal) authentication.getPrincipal();
		String username="";
		String email="";
		String attribute01="";
      
      if (principal instanceof KeycloakPrincipal) {
          KeycloakPrincipal<?> kPrincipal = (KeycloakPrincipal<?>) principal;
          IDToken token = kPrincipal.getKeycloakSecurityContext().getIdToken();
          //System.out.println("Token: " + token);
          
          Map<String, Object> customClaims = token.getOtherClaims();
          
          if (customClaims.containsKey("email")) {
              email = String.valueOf(customClaims.get("email"));
          }        
          if (customClaims.containsKey("username")) {
        	  username = String.valueOf(customClaims.get("username"));
          }
          if (customClaims.containsKey("attribute01")) {
        	  attribute01 = String.valueOf(customClaims.get("attribute01"));
          }
          model.addAttribute("username", username);
          model.addAttribute("principalid", principal.getName());
          model.addAttribute("email", email);
          model.addAttribute("attribute01", attribute01);          
      }
      
      List<Image> images = Docker.getImages();
      //System.out.println(images);     
      List<Container> containers = Docker.getContainers();
      //System.out.println(containers);
      
      model.addAttribute("docker_images", images);
      model.addAttribute("docker_containers", containers);
      
      ArrayList<String> availableOs = new ArrayList<String>();
      availableOs.add("archlinux");
      availableOs.add("opensuse-leap");
      availableOs.add("opensuse-tumbleweed");
      availableOs.add("ubuntu");
      model.addAttribute("availableOs", availableOs);
      //model.addAttribute("osChoice", new String());    
      
      return("portal");
	}
	
	@DeleteMapping("/frontend/container/delete/{id}")
	public static String deleteContainer(@PathVariable String id, HttpServletRequest request, RedirectAttributes redirectAttributes) {
	 // [Start] This block should move to a logging method. It's only job is to print user details to the console.
	     KeycloakAuthenticationToken principal = (KeycloakAuthenticationToken) request.getUserPrincipal();
	     String username= null;
	     String userid = principal.getName();
	     IDToken token = principal.getAccount().getKeycloakSecurityContext().getIdToken();		
	     Map<String, Object> customClaims = token.getOtherClaims();
	     username = String.valueOf(customClaims.get("username"));
 	 // [End]
	        
		System.out.printf("Deletion triggered for ID %s by %s (%s)", id, userid, username);
		
		Docker.deleteContainer(id);
			
		return("redirect:/portal");
	}
	
	@PostMapping(path="/frontend/container/add",consumes=MediaType.APPLICATION_FORM_URLENCODED_VALUE)
	public static String addContainer(@RequestBody MultiValueMap<String, String> body, HttpServletRequest request) {
	     KeycloakAuthenticationToken principal = (KeycloakAuthenticationToken) request.getUserPrincipal();
	     String username= null;
	     String userid = principal.getName();
	     IDToken token = principal.getAccount().getKeycloakSecurityContext().getIdToken();		
	     Map<String, Object> customClaims = token.getOtherClaims();
	     username = String.valueOf(customClaims.get("username"));
	     String osChoice = (String) body.getFirst("osChoice");
	     
	     System.out.printf("New container with OS %s requested by %s (%s)", osChoice, userid, username);
	     
	     Random rand = new Random();
	     Integer randomInt = rand.nextInt(9999999-1111);
	     Integer count = randomInt;
	     
	     //Docker.buildImage(username, osChoice, count);
	     
	     return("redirect:/portal");
	}
	
	@GetMapping(path = "/logout")
	public String logout(HttpServletRequest request) throws ServletException {
	  request.logout();
	  keycloakSessionLogout(request);
	  return "logout";
	 }
	
   private void keycloakSessionLogout(HttpServletRequest request){
      RefreshableKeycloakSecurityContext c = getKeycloakSecurityContext(request);
      KeycloakDeployment d = c.getDeployment();
      c.logout(d);
   }

   private RefreshableKeycloakSecurityContext getKeycloakSecurityContext(HttpServletRequest request){
	  return (RefreshableKeycloakSecurityContext) request.getAttribute(KeycloakSecurityContext.class.getName());
   }	
}
